﻿// Copyright 2009-2014 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace MutableData {
  class Program {
    static void Main(string[] args) {
      var rubberChicken = new Product { Name = "Rubber chicken", Price = 5.98m };
      var pulley = new Product { Name = "Pulley", Price = 2.99m };
      products = new List<Product> { rubberChicken, pulley };
      var order = new Order {
        Date = DateTime.Today,
        Lines = new List<OrderLine>{
          new OrderLine { Product = rubberChicken, Count = 3 },
          new OrderLine { Product=pulley, Count=3 }
        }
      };

      // run the real world simulator
      new Thread((ParameterizedThreadStart) RealWorldSimulator) {
        IsBackground = true
      }.Start(order);

      // run this test several times 
      for (int i = 0; i < 100; i++)
        TestOrderValue(order);

    }

    static List<Product> products;

    static void TestOrderValue(Order order) {
      try {
        if (order.GetValue( ) == (3 * 5.98m + 3 * 2.99m))
          Console.WriteLine("Everything's cool");
        else
          Console.WriteLine("Argh!");
      }
      catch (Exception ex) {
        Console.WriteLine("Caught exception: " + ex.Message);
      }
    }

    static void RealWorldSimulator(object orderObj) {
      var order = (Order) orderObj;
      while (true) {
        // do things to the order, in the way preferred by the API
        while(true)
          order.Lines.Add(new OrderLine { Product = products[0], Count = 2 });
      }
    }

  }

  public class Product {
    public string Name { get; set; }
    public decimal Price { get; set; }
  }

  public class OrderLine {
    public Product Product { get; set; }
    public int Count { get; set; }

    public decimal GetValue( ) {
      return Product.Price * Count;
    }
  }

  public class Order {
    public DateTime Date { get; set; }
    public List<OrderLine> Lines { get; set; }
    
    public decimal GetValue( ) {
      decimal result = 0;
      foreach (var line in Lines)
        result += line.GetValue( );
      return result;
    }
  }
}
